#include "FileMgr.h"

CFileMgr::CFileMgr()
{
	m_strPath = getcfgs(CFG_FILEDIR);
	m_strMusicType = getcfgs(CFG_MUSICTYPE);
	m_strListType = getcfgs(CFG_LISTTYPE);
}

CFileMgr::~CFileMgr()
{
	setcfg(CFG_FILEDIR,m_strPath.c_str());
}

int CFileMgr::GetSize()
{
	return m_vctFile.size();
}

void CFileMgr::Clear()
{
	m_vctFile.clear();
}

int CFileMgr::FindIndex(const char* lpszStr)
{
	int nRet = -1;
	std::vector<FILEITEM>::iterator iter;
	int nIndex = 0;
	for (iter = m_vctFile.begin(); iter != m_vctFile.end(); ++iter)
	{
		FILEITEM item = (*iter);
		if (0 == item.strName.compare(lpszStr))
		{
			nRet = nIndex;
			break;
		}
		nIndex ++;
	}
	return nRet;
}

bool CFileMgr::SetSel(int nIndex)
{
	const char* lpszStr = GetName(nIndex);
	if (lpszStr && (GetType(nIndex) & DT_DIR))
	{
		if (0 == strcmp(lpszStr, "../"))
		{
			size_t nPos = m_strPath.size()-1;
			if (nPos == m_strPath.rfind('/'))
			{
				m_strPath.erase(nPos,1);
			}
			nPos = m_strPath.rfind('/');
			if (nPos != std::string::npos)
			{
				m_strPath = m_strPath.substr(0,nPos+1);
			}
		}
		else
		{
			m_strPath += lpszStr;
		}
		//		log("path:%s\n",m_strPath.c_str());
		return true;
	}
	return false;
}

bool CFileMgr::CmpName(const FILEITEM& lItem, const FILEITEM& rItem)
{
	bool bRet = lItem.strName < rItem.strName;

	return bRet;
}

void CFileMgr::Load(int nType)
{
	if (nType == -1)
	{
		return ;
	}
	DIR* dirp;
	struct dirent* direntp;
	dirp = opendir(m_strPath.c_str());
	if (dirp != NULL)
	{
		FILEITEM item;
		while (true)
		{
			direntp = readdir(dirp);
			if (direntp == NULL)
			{
				break;
			}
			item.ucType = direntp->d_type;
			item.strName = direntp->d_name;
			if (direntp->d_type & DT_DIR)
			{
				item.strName += "/";
			}
			if (0 == item.strName.compare("../") && 0 == m_strPath.compare("/"))
			{

			}
			else if (0 == item.strName.substr(0,1).compare(".") && item.strName.compare("../")!=0)
			{
				//hide file such as .file dir.
			}
			else if (item.strName.compare("./")!=0)
			{
				if (direntp->d_type & DT_DIR)
				{
					item.strType = "/";
					m_vctFile.push_back(item);
				}
				else
				{
					if (0 == nType)
					{
						InsertItem(m_strMusicType,&item);
					}
					else if (1 == nType)
					{
						InsertItem(m_strListType,&item);
					}
					else if (2 == nType)
					{
						
					}
				}
			}
		}
		std::sort(m_vctFile.begin(), m_vctFile.end(), CmpName);
		closedir(dirp);
	}
}

void CFileMgr::InsertItem(const std::string& strType,const LPFILEITEM pItem)
{
	if (!pItem)
	{
		return;
	}
	if (strType.empty())	// all type file ...
	{
		pItem->strType = "*";
		m_vctFile.push_back(*pItem);
	}
	else
	{
		std::string strTmp;
		size_t nPos = pItem->strName.rfind('.');
		if (std::string::npos != nPos)
		{
			strTmp = pItem->strName.substr(nPos+1,
					pItem->strName.length()-nPos-1);
			pItem->strType = strTmp;
			strTmp = "|" + strTmp + "|";
			if (std::string::npos != strType.find(strTmp))
			{
				m_vctFile.push_back(*pItem);
			}
		}
	}
}

const char* CFileMgr::GetName(int nIndex)
{
	if (nIndex >= 0 && nIndex<(int)m_vctFile.size())
	{
		return m_vctFile[nIndex].strName.c_str();
	}
	return 0;
}

const char* CFileMgr::GetDesc(int nIndex)
{
	if (nIndex >= 0 && nIndex<(int)m_vctFile.size())
	{
		return m_vctFile[nIndex].strType.c_str();
	}
	return 0;
}

std::string CFileMgr::GetPath(int nIndex)
{
	std::string strRet;
	if (-1 == nIndex)
	{
		strRet = m_strPath;
	}
	else
	{
		const char* lpszStr = GetName(nIndex);
		if (lpszStr)
		{
			strRet = m_strPath + lpszStr;
		}
	}

	return strRet;
}

unsigned char CFileMgr::GetType(int nIndex)
{
	if (nIndex >= 0 && nIndex<(int)m_vctFile.size())
	{
		return m_vctFile[nIndex].ucType;
	}
	return DT_UNKNOWN;
}

