#include "PluginMgr.h"

CPluginMgr::CPluginMgr()
{
	m_strPluginDir = getcfgs(CFG_PLUSDIR);
}

CPluginMgr::~CPluginMgr()
{
}

int CPluginMgr::GetSize()
{
	return m_vctPlugin.size();
}

const char* CPluginMgr::GetName(int nIndex)
{
	if (nIndex >= 0 && nIndex<(int)m_vctPlugin.size())
	{
		return m_vctPlugin[nIndex].strName.c_str();
	}
	return 0;
}

const char* CPluginMgr::GetDesc(int nIndex)
{
	if (nIndex >= 0 && nIndex<(int)m_vctPlugin.size())
	{
		return m_vctPlugin[nIndex].strDesc.c_str();
	}
	return 0;
}

void CPluginMgr::Load()
{
	if (m_strPluginDir.empty())
	{
		return;
	}
	DIR* dirp;
	struct dirent* direntp;
	dirp = opendir(m_strPluginDir.c_str());
	bool bRet = false;
	if (dirp != NULL)
	{
		while (true)
		{
			direntp = readdir(dirp);
			if (direntp == NULL)
			{
				break;
			}
			if (direntp->d_type & DT_DIR)
			{
				continue;
			}
			bRet = Parse(direntp->d_name);
		}
		closedir(dirp);
	}
}

bool CPluginMgr::Parse(const char* lpszFile)
{
	if (!lpszFile)
	{
		return false;
	}
	if (0!=strcmp(lpszFile+strlen(lpszFile)-3, ".sh"))
	{
		return false;
	}
	PLUGINITEM item;
	char szPath[MAX_PATH];
	sprintf(szPath, "%s%s", PLUGIN_DIR, lpszFile);
	std::ifstream infile(szPath);
	std::string strTmp, strDesc;
	std::string strEvent;
	int nHotKey = -1;
	int nCnt = 0;
	bool bOK = false;
	while (nCnt < 10 && getline(infile, strTmp))
	{
		nCnt ++;
		if (strTmp.substr(0, 1)!="#")
		{
			break;
		}
		strTmp.erase(0,1);
		if (0==strTmp.compare("/bin/bash"))
		{
			continue;
		}
		if (0==strTmp.compare(PLUGIN_FLAG))
		{
			bOK = true;
		}
		else if (0==strTmp.find(PLUGIN_HOTKEY))
		{
			strDesc += strTmp + "\n";
			size_t LPos = strTmp.find('[');
			size_t RPos = strTmp.find(']');
			if (2==(RPos-LPos))
			{
				char ch = *strTmp.substr(LPos+1,1).c_str();
				nHotKey = ch;
			}
			LPos = strTmp.find('<');
			RPos = strTmp.find('>');
			if (RPos > LPos && LPos != std::string::npos && LPos != std::string::npos)
			{
				nHotKey = atoi(strTmp.substr(LPos+1,RPos-LPos-1).c_str());
			}
		}
		else if (0==strTmp.find(PLUGIN_EVENT))
		{
			size_t LPos = strTmp.find('[');
			if (std::string::npos != LPos)
			{
				strEvent = strTmp.substr(LPos,strTmp.length()-LPos);
			}
		}
		else
		{
			strDesc += strTmp + "\n";
		}
	}
	if (bOK)
	{
		strTmp = lpszFile;
		item.strName = strTmp.substr(0,strTmp.length()-3);
		item.strDesc = strDesc;
		item.nHotKey = nHotKey;
		item.strEvent = strEvent;
		m_vctPlugin.push_back(item);

		return true;
	}
	return false;
}

int CPluginMgr::Run(int nIndex, const char* lpszParam)
{
	int nRet = -1;
	char szCmd[128];
	memset(szCmd,0,128);
	if (!lpszParam)
	{
		if (m_vctPlugin[nIndex].strParam.empty())
		{
			sprintf(szCmd, "sh %s%s.sh", PLUGIN_DIR,
					m_vctPlugin[nIndex].strName.c_str());
		}
		else
		{
			nRet = Run(nIndex, m_vctPlugin[nIndex].strParam.c_str());
		}
	}
	else
	{
		sprintf(szCmd, "sh %s%s.sh '%s'", PLUGIN_DIR,
				m_vctPlugin[nIndex].strName.c_str(), lpszParam);
	}
	if (strlen(szCmd) > 0)
	{
		nRet = system(szCmd);
	}
	return nRet;
}

int CPluginMgr::OnHotKey(int ch)
{
	if (ch != -1)
	{
		std::vector<PLUGINITEM>::iterator iter;
		int nCnt = 0;
		for (iter=m_vctPlugin.begin(); iter!=m_vctPlugin.end(); ++iter)
		{
			if (iter->nHotKey == ch)
			{
				return nCnt;
			}
			nCnt ++;
		}
	}

	return -1;
}

int CPluginMgr::OnEvent(const char* lpszEvent, const char* lpszParam)
{
	std::vector<PLUGINITEM>::iterator iter;
	char szEvent[32];
	sprintf(szEvent, "[%s]", lpszEvent);
	for (iter=m_vctPlugin.begin(); iter!=m_vctPlugin.end(); ++iter)
	{
		if (std::string::npos != iter->strEvent.find(szEvent) )
		{
			iter->strParam = lpszParam;
		}
	}
	return 0;
}
