#include "UIMgr.h"

#define _ESC_KEY	27//'`'
CUIMgr::CUIMgr()
{
	setlocale(LC_ALL, "");
	initscr();

	cbreak();
	noecho();
	curs_set(0);

	m_bRun = false;
	m_bPainLock = false;
	m_bPlaying = false;
	m_bPause = true;
	m_bPluginLock = false;
	m_bModeLock = false;
	m_bFileLock = false;
	m_bAboutLock = false;
	m_bMsgBoxLock = false;
	m_bTipScr = false;
	m_nFileType = -1;
	m_nCurSel = 0;
	m_nLrcLight = getcfgi(CFG_LRCLIGHT);
	m_nVolum = getcfgi(CFG_VOLUME);
	m_strLrcDir = getcfgs(CFG_LRCDIR);
	m_nCurMode = getcfgi(CFG_PLAYMODE);
	m_nKeyMode = getcfgi(CFG_KEYMODE);
	m_tSTime = m_tRTime = time(NULL);
	for (int i = 0; i < WIN_CNT; ++i)
	{
		m_wins[i] = 0;
		m_panels[i] = 0;
	}
	m_menu = m_modemenu = m_plusmenu = 0;
	Layout();
}

CUIMgr::~CUIMgr()
{
	free_menu(m_modemenu);
	for (int i = 0; i < 5; ++i)
	{
		free_item(m_modeitems[i]);
	}
	free(m_modeitems);
	DestroyListMenu();
	endwin();
	setcfg(CFG_KEYMODE,m_nKeyMode);
	setcfg(CFG_VOLUME,m_nVolum);
	setcfg(CFG_PLAYMODE,m_nCurMode);
	setcfg(CFG_LRCDIR,m_strLrcDir.c_str());
}

void CUIMgr::DelayRun()
{
	std::vector<DELAYEVENT>::iterator iter;
	std::string strEvent;
	std::string strParam;
	for (iter = m_vctDelayEvent.begin(); iter != m_vctDelayEvent.end(); ++iter)
	{
		strEvent = iter->strEvent;
		strParam = iter->strParam;
		if (strEvent.compare(NOTIFY_LDLST) == 0)
		{
			if (strParam.empty())
			{
				CreateListMenu(m_wins[2], getpath("music.lst"));
			}
			else
			{
				CreateListMenu(m_wins[2], strParam.c_str());
			}
		}
	}
	m_vctDelayEvent.clear();
}

int CUIMgr::OnMsg(const char* lpszMsg, WPARAM wParam, LPARAM lParam)
{
	if (0 == strcmp(lpszMsg, MSG_NAME))
	{
		switch (wParam)
		{
		case 0://name
		{
			m_strName = (const char*)lParam;
			mvwhline(m_wins[1],1, 1, ' ', STD_WID-2);
			mvwprintw(m_wins[1], 1, 1, m_strName.c_str());
			wrefresh(m_wins[1]);
			// as loadfile send this msg,when recv,means playing ...
			m_bPlaying = true;
			m_bPause = false;
			m_fTPos = 0.0f;
			m_fTLen = 0.0f;
			m_strInfo = "";
		}
			break;
		case 1://samples
		case 2://rate
		{
			std::string strTmp = (const char*)lParam;
			m_strInfo += ("[" + strTmp + "]");
			mvwhline(m_wins[1],2, 1, ' ', STD_WID-2);
			mvwprintw(m_wins[1], 2, 1, m_strInfo.c_str());
			wrefresh(m_wins[1]);
		}
			break;
		case 3://tlen
		{
			if (m_fTLen >= 0.0f)
			{
				float* pfLen = (float*)lParam;
				m_fTLen = *pfLen;
			}
		}
			break;
		default:
			break;
		}

	}
	else if (0 == strcmp(lpszMsg, MSG_TPOS))
	{
		m_tRTime = time(NULL);
		float* pfPos = (float*)lParam;
		m_fTPos = *pfPos;
	}
	else if (0 == strcmp(lpszMsg, MSG_NOTIFY))
	{
		const char* lpszType = (const char*)wParam;
		const char* lpszParam = (const char*)lParam;
		if (lpszType)
		{
			if (0==strcmp(lpszType, NOTIFY_LDLRC))
			{
				std::string str = GetLrcPath(
						m_vctMenu[m_nCurSel].strPath.c_str(), 2);
				if (!str.empty())
				{
					m_nLastLrcIndex = -1;
					m_LrcMgr.LoadFile(str.c_str());
				}
			}
			else if (0==strcmp(lpszType, NOTIFY_LDLST))
			{
				DELAYEVENT event;
				event.strEvent = NOTIFY_LDLST;
				if (lpszParam)
				{
					event.strParam = lpszParam;
				}
				m_vctDelayEvent.push_back(event);
			}
			else if (0==strcmp(lpszType, NOTIFY_LDURL))
			{
				SendMgrMsg(MSG_URL,0,lParam);
			}
		}
	}
	else if (0==strcmp(lpszMsg, MSG_EVENT))
	{
		m_bPainLock = true;
		echo();
		curs_set(2);
		def_prog_mode();
		endwin();
		m_PluginMgr.OnEvent((const char*)wParam, (const char*)lParam);
		reset_prog_mode();
		noecho();
		curs_set(0);
		refresh();
		m_bPainLock = false;
	}
	else if (0 == strcmp(lpszMsg, MSG_QUIT))
	{
		m_bRun = false;
	}
	return 0;
}

void CUIMgr::Layout()
{
	ReSize(true);

	keypad(stdscr, TRUE);
	mousemask(ALL_MOUSE_EVENTS, NULL);

	pthread_t p_thread;
	pthread_create(&p_thread, 0, LoopProc, this);
}

void CUIMgr::ReSize(bool bForce)
{
	if (m_nLINES != LINES || m_nCOLS != COLS || bForce)
	{
		m_nLINES = LINES;
		m_nCOLS = COLS;
		if (m_nLINES < STD_HEG + 8 || COLS < STD_WID + 4)
		{
			m_bTipScr = true;
			for (int i = 0; i < WIN_CNT; ++i)
			{
				hide_panel(m_panels[i]);
			}
			mvprintw(0, 0, _VER_DESC,_VERSION,"press resize the console ...\n");
			update_panels();
			doupdate();
			return;
		}
		m_bTipScr = false;
		WINDOW *wins[WIN_CNT];
		wins[0] = newwin(m_nLINES, m_nCOLS, 0, 0);
		wins[1] = newwin(STD_HEG, STD_WID, 1, 1);
		wins[2] = newwin(m_nLINES-STD_HEG-2, STD_WID, STD_HEG+1, 1);
		wins[3] = newwin(m_nLINES-2, m_nCOLS-STD_WID-2, 1, STD_WID+1);
		wins[4] = newwin(STD_HEG,m_nCOLS-4,(m_nLINES-STD_HEG)/2,2);
		for (int i = 0; i < WIN_CNT; ++i)
		{
			if (ismode("text"))
			{
				wborder(wins[i], '|', '|', '-', '-', '+', '+', '+', '+');
			}
			else
			{
				box(wins[i], 0, 0);
			}
		}
		mvwhline(wins[1],3, 2, '-', 21);
		mvwaddch(wins[1],3, 1, '[');
		mvwaddch(wins[1],3, 23, ']');
		mvwaddch(wins[1],3, m_nVolum/5+3-1, '$');
		mvwprintw(wins[1], 3, 24, "(%3d%%)", m_nVolum);
		mvwaddstr(wins[1],3, 30,"(0:00:00)");
		mvwhline(wins[1],4, 1, ':', STD_WID-2);
		mvwaddstr(wins[1], 5, 1, "[ |< ]  [PAUS]  [STOP]  [ >| ]  [MUTE]");

		mvwhline(wins[1],1, 1, ' ', STD_WID-2);
		mvwprintw(wins[1], 1, 1, m_strName.c_str());
		mvwhline(wins[1],2, 1, ' ', STD_WID-2);
		mvwprintw(wins[1], 2, 1, m_strInfo.c_str());

		if (m_menu)
		{
			unpost_menu(m_menu);
			set_menu_win(m_menu, wins[2]);
			set_menu_sub(m_menu, derwin(wins[2], m_nLINES-STD_HEG-7, 
			STD_WID-2, 2, 1));
			set_menu_format(m_menu, m_nLINES-STD_HEG-7, 1);
			if (!m_bPluginLock)
			{
				post_menu(m_menu);
			}
		}
		else
		{
			CreateListMenu(wins[2], getpath("music.lst"));
		}

		if (m_plusmenu)
		{
			unpost_menu(m_plusmenu);
			set_menu_win(m_plusmenu, wins[2]);
			set_menu_sub(m_plusmenu, derwin(wins[2], m_nLINES-STD_HEG-7, 
			STD_WID-2, 2, 1));
			set_menu_format(m_plusmenu, m_nLINES-STD_HEG-7, 1);
			if (m_bPluginLock)
			{
				post_menu(m_plusmenu);
			}
		}
		else
		{
			CreatePluginMenu(wins[2]);
		}

		if (m_modemenu)
		{
			unpost_menu(m_modemenu);
			set_menu_win(m_modemenu, wins[2]);
			set_menu_sub(m_modemenu, derwin(wins[2], 1, STD_WID-2, m_nLINES
					-STD_HEG-2-3, 1));
			if (m_bModeLock)
			{
				post_menu(m_modemenu);
			}
		}
		else
		{
			CreateModeMenu(wins[2]);
		}

		if (m_msgmenu)
		{
			unpost_menu(m_msgmenu);
			set_menu_win(m_msgmenu, wins[4]);
			set_menu_sub(m_msgmenu, derwin(wins[4], 1, 16, 5, (m_nCOLS-4 - 16)
					/2));
			post_menu(m_msgmenu);
		}
		else
		{
			CreateMsgMenu(wins[4]);
		}

		if (m_filemenu)
		{
			unpost_menu(m_filemenu);
			set_menu_win(m_filemenu, wins[3]);
			set_menu_sub(m_filemenu, derwin(wins[3], m_nLINES-6, m_nCOLS
					-STD_WID-2-2, 2, 1));
			set_menu_format(m_filemenu, m_nLINES-6, 1);
			if (m_bFileLock)
			{
				post_menu(m_filemenu);
				mvwhline(wins[3],1, 1, ' ', m_nCOLS-STD_WID-2-2);
				mvwprintw(wins[3], 1, 1, "dir:%s", m_FileMgr.GetPath(-1).c_str());
			}
		}
		else
		{
			CreateFileMenu(wins[3]);
		}

		mvwaddstr(wins[2], 1, 1, "[ ++ ]  [ -- ]  [ UP ]  [ DN ]  [ CL ]");
		mvwprintw(wins[2], m_nLINES-STD_HEG-4, 1,
				"[%s]==[LIST]==[LRCD]==[PLUS]==[%s]",
				m_strMode[m_nCurMode].c_str(), m_nKeyMode==0 ? "CKEY" : "MKEY");

		m_LrcMgr.SetHight(m_nLINES-8);

		mvwhline(wins[4],1, 1, '=', m_nCOLS-6);
		mvwhline(wins[4],3, 1, ' ', m_nCOLS-6);
		mvwhline(wins[4],4, 1, ' ', m_nCOLS-6);
		if (!m_strMsgCaption.empty())
		{
			mvwaddstr(wins[4], 1, (m_nCOLS-4-2-m_strMsgCaption.length() )/2, m_strMsgCaption.c_str());
		}
		if (!m_strMsgInfo.empty())
		{
			mvwaddstr(wins[4], 3, (m_nCOLS-4-2-m_strMsgInfo.length())/2, m_strMsgInfo.c_str());
		}

		if (m_panels[0])
		{
			for (int i = 0; i < WIN_CNT; ++i)
			{
				replace_panel(m_panels[i], wins[i]);
				delwin(m_wins[i]);
				m_wins[i]=wins[i];
			}
		}
		else
		{
			for (int i = 0; i < WIN_CNT; ++i)
			{
				m_panels[i] = new_panel(wins[i]);
				m_wins[i]=wins[i];
			}
		}

		if (!m_bMsgBoxLock)
		{
			hide_panel(m_panels[4]);
		}

		for (int i = 0; i < WIN_CNT-1; ++i)
		{
			show_panel(m_panels[i]);
		}

		update_panels();
		doupdate();

		if (m_bPluginLock)
		{
			int nIndex = item_index(current_item(m_plusmenu));
			PrintPlugin(m_PluginMgr.GetDesc(nIndex));
		}
		else
		{
			RefreshLrc(m_fTPos, true);
		}
	}
}

int CUIMgr::MsgBox(const char* lpszCaption, const char* lpszMsg, int nType)
{
	int nRet = -1;
	m_bMsgBoxLock = true;
	mvwhline(m_wins[4],1, 1, '=', m_nCOLS-6);
	mvwhline(m_wins[4],3, 1, ' ', m_nCOLS-6);
	mvwhline(m_wins[4],4, 1, ' ', m_nCOLS-6);
	if (lpszCaption)
	{
		m_strMsgCaption = lpszCaption;
		mvwaddstr(m_wins[4], 1, (m_nCOLS-4-2-strlen(lpszCaption))/2, lpszCaption);
	}
	if (lpszMsg)
	{
		m_strMsgInfo = lpszMsg;
		mvwaddstr(m_wins[4], 3, (m_nCOLS-4-2-strlen(lpszMsg))/2, lpszMsg);
	}
	menu_driver(m_msgmenu, REQ_RIGHT_ITEM);
	show_panel(m_panels[4]);
	update_panels();
	doupdate();
	bool bNext = true;
	int ch =-1;
	do
	{
		ch = getch();
		switch (ch)
		{
		case ' ':
		case 10:
		{
			bNext = false;
		}
			break;
		case KEY_LEFT:
		{
			menu_driver(m_msgmenu, REQ_LEFT_ITEM);
			wrefresh(m_wins[4]);
		}
			break;
		case KEY_RIGHT:
		{
			menu_driver(m_msgmenu, REQ_RIGHT_ITEM);
			wrefresh(m_wins[4]);
		}
			break;
		default:
		{

		}
			break;
		}
		if (10==ch || ' '==ch)
		{
			nRet = item_index(current_item(m_msgmenu));
		}
	} while (bNext);
	hide_panel(m_panels[4]);
	update_panels();
	doupdate();
	m_bMsgBoxLock = false;
	return nRet;
}

int CUIMgr::getch2(int& ch)
{
	ch= getch();
	int chex = 0;
	if (iskey(VKEY_NOUSE,ch))
	{
		chex = -1;
	}
	else if (iskey(VKEY_KEYMODE,ch))
	{
		chex = 'k';
	}
	else if (iskey(VKEY_QUIT,ch))
	{
		chex = 'q';
	}
	else if (iskey(VKEY_REFRESH,ch))
	{
		chex = 'f';
	}
	else if (iskey(VKEY_MODE,ch))
	{
		chex = 'd';
	}
	else if (iskey(VKEY_LIST,ch))
	{
		chex = 'l';
	}
	else if (iskey(VKEY_LRC,ch))
	{
		chex = 'r';
	}
	else if (iskey(VKEY_PLUS,ch))
	{
		chex = 'g';
	}
	else if (iskey(VKEY_ABOUT,ch))
	{
		chex = 'a';
	}
	else if (iskey(VKEY_PREV,ch))
	{
		chex = ',';
	}
	else if (iskey(VKEY_NEXT,ch))
	{
		chex = '.';
	}
	else if (iskey(VKEY_PAUSE,ch))
	{
		chex = 'p';
	}
	else if (iskey(VKEY_STOP,ch))
	{
		chex = 's';
	}
	else if (iskey(VKEY_MUTE,ch))
	{
		chex = 'm';
	}
	else if (iskey(VKEY_ADD,ch))
	{
		chex = '=';
	}
	else if (iskey(VKEY_DEL,ch))
	{
		chex = '-';
	}
	else if (iskey(VKEY_CLR,ch))
	{
		chex = 'c';
	}
	else if (iskey(VKEY_VOLUP,ch))
	{
		chex = ']';
	}
	else if (iskey(VKEY_VOLDN,ch))
	{
		chex = '[';
	}
	else if (iskey(VKEY_BLANK,ch))
	{
		chex = ' ';
	}
	else if (iskey(VKEY_ENTER,ch))
	{
		chex = 10;
	}
	else if (iskey(VKEY_ESC,ch))
	{
		chex = _ESC_KEY;
	}
	else if (ch >= KEY_CODE_YES && ch <= KEY_MAX)
	{
		chex = ch;
	}
	return chex;
}

void* CUIMgr::LoopProc(void* pData)
{
	CUIMgr* pMgr = (CUIMgr*)pData;
	if (pMgr)
	{
		pMgr->InputLoop();
	}
	return 0;
}

void CUIMgr::InputLoop()
{
	m_bRun = true;
	int c;
	int ch;
	while ((c = getch2(ch)) != 'q')
	{
		if (m_bTipScr)
		{
			doupdate();
			continue;
		}
		switch (ch)
		{
		case 'k':
		{
			Command(eUCKeyMode);
		}
			break;
		case ' ':
		case 'p':
		{
			Command(eUCPlay);
		}
			break;
		case 'm':
		{
			Command(eUCMute);
		}
			break;

		default:
		{
			if (c == KEY_MOUSE)
			{
				MEVENT event;
				if (getmouse(&event) == OK)
				{
					if ((event.bstate & BUTTON1_CLICKED) || (event.bstate
							& BUTTON1_DOUBLE_CLICKED))
					{
						MouseHint(&event);
					}
					else if ((event.bstate & BUTTON2_PRESSED))
					{
						if (m_menu)
						{
							menu_driver(m_menu, REQ_SCR_DPAGE);
							wrefresh(m_wins[2]);
						}
					}
					else if (event.bstate & BUTTON4_PRESSED)
					{
						if (m_menu)
						{
							menu_driver(m_menu, REQ_SCR_UPAGE);
							wrefresh(m_wins[2]);
						}
					}
				}
				continue;
			}
			if (0 == m_nKeyMode)
			{
				int nIndex = m_PluginMgr.OnHotKey(ch);
				if (nIndex != -1)
				{
					SaveList();//avoid changed by outside.
					m_bPainLock = true;
					echo();
					curs_set(2);
					def_prog_mode();
					endwin();
					m_PluginMgr.Run(nIndex);
					reset_prog_mode();
					noecho();
					curs_set(0);
					refresh();
					m_bPainLock = false;
					DelayRun();
					continue;
				}
				switch (c)
				{
				case '-':
				{
					Command(eUCDelItem);
				}
					break;
				case '=':
				{
					Command(eUCAddItem);
				}
					break;
				case 'a':
				{
					Command(eUCAbout);
				}
					break;
				case 'c':
				{
					Command(eUCClrItem);
				}
					break;
				case 'f':
				{
					ReSize(true);
				}
					break;
				case 'g':
				{
					Command(eUCPlus);
				}
					break;
				case ',':
				{
					Command(eUCPrev);
				}
					break;
				case '.':
				{
					Command(eUCNext);
				}
					break;

				case 's':
				{
					Command(eUCStop);
				}
					break;
				case 'd':
				{
					Command(eUCMode);
				}
					break;
				case 'l':
				{
					Command(eUCList);
				}
					break;
				case 'r':
				{
					Command(eUCLrcd);
				}
					break;
				case '[':
				{
					if (m_nVolum > 0)
					{
						m_nVolum -= 5;
						mvwhline(m_wins[1],3, 2, '-', 21);
						mvwaddch(m_wins[1],3, 1, '[');
						mvwaddch(m_wins[1],3, 23, ']');
						mvwaddch(m_wins[1],3, m_nVolum/5+3-1, '$');
						mvwprintw(m_wins[1], 3, 24, "(%3d%%)", m_nVolum);
						wrefresh(m_wins[1]);
						if (m_nVolum == 0)
						{
							SendMgrMsg(MSG_VOLUM,0,(LPARAM)1);
						}
						else
						{
							SendMgrMsg(MSG_VOLUM,0,(LPARAM)m_nVolum);
						}
						usleep(1000*1000);
					}
				}
					break;
				case ']':
				{
					if (m_nVolum < 100)
					{
						m_nVolum += 5;
						mvwhline(m_wins[1],3, 2, '-', 21);
						mvwaddch(m_wins[1],3, 1, '[');
						mvwaddch(m_wins[1],3, 23, ']');
						mvwaddch(m_wins[1],3, m_nVolum/5+3-1, '$');
						mvwprintw(m_wins[1], 3, 24, "(%3d%%)", m_nVolum);
						wrefresh(m_wins[1]);
						if (m_nVolum == 0)
						{
							SendMgrMsg(MSG_VOLUM,0,(LPARAM)1);
						}
						else
						{
							SendMgrMsg(MSG_VOLUM,0,(LPARAM)m_nVolum);
						}
						usleep(1000*1000);
					}
				}
					break;

				case 10://ENTER
				{
					if (m_menu)
					{
						int nIndex = item_index(current_item(m_menu));
						m_nCurSel = nIndex;
						SendMgrMsg(MSG_FILE,0,(LPARAM)m_vctMenu[m_nCurSel].strPath.c_str());
					}
				}
					break;

				case KEY_LEFT:
				{
					Command(eUCUpItem);
				}
					break;
				case KEY_RIGHT:
				{
					Command(eUCDnItem);
				}
					break;
				case KEY_DOWN:
				{
					if (m_menu)
					{
						menu_driver(m_menu, (item_index(current_item(m_menu))
								>= item_count(m_menu)-1) ? REQ_FIRST_ITEM
								: REQ_DOWN_ITEM);
						wrefresh(m_wins[2]);
					}
				}
					break;
				case KEY_UP:
				{
					if (m_menu)
					{
						menu_driver(m_menu, (item_index(current_item(m_menu))
								<= 0) ? 
						REQ_LAST_ITEM : REQ_UP_ITEM);
						wrefresh(m_wins[2]);
					}
				}
					break;
				case KEY_NPAGE:
				{
					if (m_menu)
					{
						menu_driver(m_menu, REQ_SCR_DPAGE);
						wrefresh(m_wins[2]);
					}
				}
					break;
				case KEY_PPAGE:
				{
					if (m_menu)
					{
						menu_driver(m_menu, REQ_SCR_UPAGE);
						wrefresh(m_wins[2]);
					}
				}
					break;
				case KEY_HOME:
				{
					menu_driver(m_menu, REQ_FIRST_ITEM);
					wrefresh(m_wins[2]);
				}
					break;
				case KEY_END:
				{
					menu_driver(m_menu, REQ_LAST_ITEM);
					wrefresh(m_wins[2]);
				}
					break;

				case 0:
				{
					// ...
				}
					break;
				default:
				{
				}
					break;
				}
			}
			else // mplayer key mode ...
			{
				switch (ch)
				{
				case 10:
				{
					Command(eUCNext);
				}
					break;
				case KEY_LEFT:
				{
					SendMgrMsg(MSG_STRCMD,0,(LPARAM)"seek -10 0");
				}
					break;
				case KEY_RIGHT:
				{
					SendMgrMsg(MSG_STRCMD,0,(LPARAM)"seek 10 0");
				}
					break;
				case KEY_UP:
				{
					SendMgrMsg(MSG_STRCMD,0,(LPARAM)"seek 60 0");
				}
					break;
				case KEY_DOWN:
				{
					SendMgrMsg(MSG_STRCMD,0,(LPARAM)"seek -60 0");
				}
					break;
				case KEY_PPAGE:
				{
					SendMgrMsg(MSG_STRCMD,0,(LPARAM)"seek 600 0");
				}
					break;
				case KEY_NPAGE:
				{
					SendMgrMsg(MSG_STRCMD,0,(LPARAM)"seek -600 0");
				}
					break;
				case KEY_HOME:
				{
					Command(eUCPrev);
				}
					break;
				case KEY_END:
				{
					Command(eUCNext);
				}
					break;
				case KEY_IC:
				{
					Command(eUCAddItem);
				}
					break;
				case KEY_DC:
				{
					Command(eUCDelItem);
				}
					break;
				default:
				{
					SendMgrMsg(MSG_CHAR,0,(LPARAM)&ch);
				}
					break;
				}
			}
		}
			break;
		}

		doupdate();
	}
	SaveList();
	m_bRun = false;
}

void CUIMgr::SaveList(const char* lpszFile)
{
	const char* lpszStr = lpszFile ? lpszFile : getpath("music.lst");
	FILE* pf = fopen(lpszStr, "w");
	if (pf)
	{
		for (size_t i = 0; i < m_vctMenu.size() ; ++i)
		{
			fprintf(pf, "%s\n", m_vctMenu[i].strPath.c_str());
		}
		fclose(pf);
	}
}

void CUIMgr::PaintLoop()
{
	int nCnt = 0;
	while (m_bRun)
	{
		if (!m_bPainLock)
		{
			nCnt ++;
			if (nCnt % 5) //for normal.
			{
				ReSize();
			}
			if (m_bPlaying && !m_bPause) //playing now ...
			{
				m_fTPos += 0.2f;
				if (nCnt % 10 && !m_bMsgBoxLock && !m_bTipScr) //for normal.
				{
					mvwprintw(m_wins[1], 3, 30, "(%s)", Float2Time(m_fTPos).c_str());
					wrefresh(m_wins[1]);
					RefreshLrc(m_fTPos);
				}
				if (nCnt % 10 == 0)//2 sec for ping ...
				{
					if (!CheckAutoNext())
					{
						if (m_fTLen > 0.0f)
						{
							int nPer = (int)((m_fTPos/m_fTLen)
									*(float)(STD_WID-2));
							mvwhline(m_wins[1],4, 1, ':', STD_WID-2);
							mvwhline(m_wins[1],4, 1, '>', nPer);
						}
						m_tSTime = time(NULL);
						SendMgrMsg(MSG_TPOS,0,0);
					}
				}
			}
			if (nCnt >= 100)
			{
				nCnt = 0;
			}
			usleep(200*1000);
		}

	}
}

void CUIMgr::MouseHint(MEVENT* event)
{
	if (!event)
	{
		return;
	}
	int x = event->x;
	int y = event->y;
	if (y >= STD_HEG+3 && y <= m_nLINES-5)
	{
		if (x >= 4 && x <= STD_WID-2)
		{
			int nOffset = y - (STD_HEG+3);
			int nIndex = top_row(m_menu) + nOffset;
			MenuToSel(nIndex);
			wrefresh(m_wins[2]);
			if (event->bstate & BUTTON1_DOUBLE_CLICKED)
			{
				m_nCurSel = nIndex;
				SendMgrMsg(MSG_FILE,0,(LPARAM)m_vctMenu[m_nCurSel].strPath.c_str());
			}
		}
	}
	if ((event->bstate & BUTTON1_CLICKED)==0)
	{
		return;
	}
	if (y == m_nLINES-3)
	{
		if (x >= 3 && x<= 7) //mode
		{
			Command(eUCMode);
		}
		else if (x>=11 && x<=15) //list
		{
			Command(eUCList);
		}
		else if (x>= 19 && x<=23) //lrc dir
		{
			Command(eUCLrcd);
		}
		else if (x>= 27 && x<=31) //plugin
		{
			Command(eUCPlus);
		}
		else if (x>= 35 && x<=39) //clr
		{
			Command(eUCKeyMode);
		}
	}
	else if (y == STD_HEG+2)
	{
		if (x >= 3 && x <= 7) //ADD
		{
			Command(eUCAddItem);
		}
		else if (x>=11 && x<=15) //DEL
		{
			Command(eUCDelItem);
		}
		else if (x>= 19 && x<=23) //UP
		{
			Command(eUCUpItem);

		}
		else if (x>= 27 && x<=31) //DN
		{
			Command(eUCDnItem);

		}
		else if (x>= 35 && x<=39) //clr
		{
			Command(eUCClrItem);
		}
	}
	else if (y == 4)
	{
		if (x >= 3 && x<=23) //volum
		{
			m_nVolum = (x-3)*5;
			mvwhline(m_wins[1],3, 2, '-', 21);
			mvwaddch(m_wins[1],3, x-1, '$');
			mvwprintw(m_wins[1], 3, 24, "(%3d%)", m_nVolum);
			wrefresh(m_wins[1]);
			if (m_nVolum == 0)
			{
				SendMgrMsg(MSG_VOLUM,0,(LPARAM)1);
			}
			else
			{
				SendMgrMsg(MSG_VOLUM,0,(LPARAM)m_nVolum);
			}

		}
	}
	else if (y == 5)
	{
		if (x >= 2 && x <= STD_WID + 2) //pos
		{
			int nPercent = (x-2)*100/STD_WID;
			SendMgrMsg(MSG_SETPOS,0,(LPARAM)nPercent);
		}
	}
	else if (y== 6)
	{
		if (x >= 3 && x <= 7) //prev
		{
			Command(eUCPrev);
		}
		else if (x>=11 && x<=15) //play
		{
			Command(eUCPlay);
		}
		else if (x>= 19 && x<=23) //stop
		{
			Command(eUCStop);
		}
		else if (x>= 27 && x<=31) //next
		{
			Command(eUCNext);
		}
		else if (x>= 35 && x<=39) //mute
		{
			Command(eUCMute);
		}
	}
}

void CUIMgr::Command(int nCmd)
{
	switch (nCmd)
	{
	case eUCKeyMode:
	{
		m_nKeyMode = !m_nKeyMode;
		mvwprintw(m_wins[2], m_nLINES-STD_HEG-4, STD_WID - 7, "[%s]",
				m_nKeyMode == 0 ? "CKEY" : "MKEY");
		wrefresh(m_wins[2]);
	}
		break;
	case eUCAbout:
	{
		PrintAbout();
	}
		break;
	case eUCClrItem:
	{
		if (0 == MsgBox("[Confirm]", "Clear All Item?"))
		{
			ITEM** ppItem = menu_items(m_menu);
			unpost_menu(m_menu);
			int nSize = item_count(m_menu);
			ITEM* pItem = 0;
			for (int i = 0; i < nSize; ++i)
			{
				pItem = ppItem[i];
				free_item(pItem);
			}
			ppItem[0]=0;
			set_menu_items(m_menu, ppItem);
			post_menu(m_menu);
			m_vctMenu.clear();
			m_nCurSel = 0;
			wrefresh(m_wins[2]);
		}
	}
	break;
	case eUCUpItem:
	{
		if (m_vctMenu.size() <= 1)
		{
			break;
		}
		int nIndex = item_index(current_item(m_menu));
		if (nIndex > 0)
		{
			nIndex --;
			ITEM** ppItem = menu_items(m_menu);
			ITEM* pItem1 = ppItem[nIndex];
			ITEM* pItem2 = ppItem[nIndex+1];
			ppItem[nIndex] = pItem2;
			ppItem[nIndex+1] = pItem1;
			MENUITEM item = m_vctMenu[nIndex];
			m_vctMenu[nIndex] = m_vctMenu[nIndex+1];
			m_vctMenu[nIndex+1] = item;
			unpost_menu(m_menu);
			set_menu_items(m_menu, ppItem);
			post_menu(m_menu);
			m_nCurSel = nIndex;
			MenuToSel(m_nCurSel);
			wrefresh(m_wins[2]);
		}
	}
	break;
	case eUCDnItem:
	{
		if (m_vctMenu.size() <= 1)
		{
			break;
		}
		int nIndex = item_index(current_item(m_menu));
		if (nIndex < item_count(m_menu)-1 && nIndex >= 0)
		{
			ITEM** ppItem = menu_items(m_menu);
			ITEM* pItem1 = ppItem[nIndex];
			ITEM* pItem2 = ppItem[nIndex+1];
			ppItem[nIndex] = pItem2;
			ppItem[nIndex+1] = pItem1;
			MENUITEM item = m_vctMenu[nIndex];
			m_vctMenu[nIndex] = m_vctMenu[nIndex+1];
			m_vctMenu[nIndex+1] = item;
			unpost_menu(m_menu);
			set_menu_items(m_menu, ppItem);
			post_menu(m_menu);
			m_nCurSel = nIndex+1;
			MenuToSel(m_nCurSel);
			wrefresh(m_wins[2]);
		}
	}
	break;
	case eUCAddItem:
	{
		m_bFileLock = true;
		ClearLrc();
		m_nFileType = 0;
		CreateFileMenu(m_wins[3]);
		post_menu(m_filemenu);
		mvwhline(m_wins[3],1, 1, ' ', m_nCOLS-STD_WID-2-2);
		mvwprintw(m_wins[3], 1, 1, "dir:%s", m_FileMgr.GetPath(-1).c_str());
		wrefresh(m_wins[3]);
		int nIndex = 0;
		int ch, chTmp;
		bool bOK = false;
		bool bNext = true;
		do
		{
			chTmp = 0;
			bOK = false;
			bNext = true;
			do
			{
				ch = getch();
				switch (ch)
				{
					case _ESC_KEY:
					{
						bNext = false;
					}
					case KEY_BACKSPACE:
					case 10://enter

					{
						bOK = true;
					}
					break;
					case ' ':
					{
						nIndex = item_index(current_item(m_filemenu));
						if (m_FileMgr.GetType(nIndex) & DT_DIR)
						{
							if (strcmp(m_FileMgr.GetName(nIndex),"../") == 0)
							{
								menu_driver(m_filemenu, REQ_DOWN_ITEM);
								ITEM **items;
								items = menu_items(m_filemenu);
								const char* pDesc = 0;
								for (int i = 0; i < item_count(m_filemenu); ++i)
								{
									pDesc = item_description(items[i]);
									if (0 != strcmp(pDesc,"/"))
									{
										set_item_value(items[i],!item_value(items[i]));
									}

								}
								wrefresh(m_wins[3]);
							}
						}
						else
						{
							menu_driver(m_filemenu, REQ_TOGGLE_ITEM);
							wrefresh(m_wins[3]);
						}
					}
					break;
					case KEY_UP:
					{
						menu_driver(m_filemenu, REQ_UP_ITEM);
						wrefresh(m_wins[3]);
					}
					break;
					case KEY_DOWN:
					{
						menu_driver(m_filemenu, REQ_DOWN_ITEM);
						wrefresh(m_wins[3]);
					}
					break;
					default:
					{
						if (ch >= 'a' && ch <= 'z')
						{
							if (chTmp == ch)
							{
								menu_driver(m_filemenu, REQ_NEXT_MATCH);
							}
							else
							{
								menu_driver(m_filemenu, REQ_CLEAR_PATTERN);
								menu_driver(m_filemenu, ch);
								chTmp = ch;
							}
							wrefresh(m_wins[3]);
						}
					}
					break;
				}
			}while (!bOK);
			if (10 == ch || KEY_BACKSPACE == ch)
			{
				nIndex = KEY_BACKSPACE == ch?
				m_FileMgr.FindIndex("../"):item_index(current_item(m_filemenu));
				if (m_FileMgr.SetSel(nIndex))
				{
					menu_driver(m_filemenu, REQ_CLEAR_PATTERN);
					unpost_menu(m_filemenu);
					CreateFileMenu(m_wins[3]);
					post_menu(m_filemenu);
					mvwhline(m_wins[3],1, 1, ' ', m_nCOLS-STD_WID-2-2);
					mvwprintw(m_wins[3], 1, 1, "dir:%s", m_FileMgr.GetPath(-1).c_str());
					wrefresh(m_wins[3]);
				}
				else if (10 == ch)
				{
					ITEM **items;
					items = menu_items(m_filemenu);
					for (int i = 0; i < item_count(m_filemenu); ++i)
					{
						if (item_value(items[i]))
						{
							MENUITEM item;
							item.strName = m_FileMgr.GetName(i);
							item.strPath = m_FileMgr.GetPath(i);
							m_vctMenu.push_back(item);
							set_item_value(items[i],false);
						}
					}
					CreateListMenu(m_wins[2], 0);
					wrefresh(m_wins[3]);
				}
			}
		}while (bNext);
		menu_driver(m_filemenu, REQ_CLEAR_PATTERN);
		unpost_menu(m_filemenu);
		mvwhline(m_wins[3],1, 1, ' ', m_nCOLS-STD_WID-2-2);
		wrefresh(m_wins[3]);
		m_bFileLock = false;
		RefreshLrc(m_fTPos, true);
	}
	break;
	case eUCDelItem:
	{
		if (m_vctMenu.size() <= 0)
		{
			break;
		}
		int nIndex = item_index(current_item(m_menu));
		if (nIndex != -1)
		{
			ITEM** ppItem = menu_items(m_menu);
			ITEM* pItem = ppItem[nIndex];
			int nCnt = nIndex;
			while (nCnt < item_count(m_menu)-1)
			{
				ppItem[nCnt] = ppItem[nCnt+1];
				m_vctMenu[nCnt] = m_vctMenu[nCnt+1];
				nCnt ++;
			}
			ppItem[nCnt] = ppItem[nCnt+1];
			m_vctMenu.pop_back();
			unpost_menu(m_menu);
			free_item(pItem);
			set_menu_items(m_menu, ppItem);
			post_menu(m_menu);
			m_nCurSel = nIndex;
			MenuToSel(m_nCurSel);
			wrefresh(m_wins[2]);
		}
	}
	break;
	case eUCPlus:
	{
		if (m_PluginMgr.GetSize() <= 0)
		{
			break;
		}
		m_bPluginLock = true;
		unpost_menu(m_menu);
		post_menu(m_plusmenu);
		wrefresh(m_wins[2]);
		int nIndex = 0;
		nIndex = item_index(current_item(m_plusmenu));
		PrintPlugin(m_PluginMgr.GetDesc(nIndex));
		int ch;
		bool bOK = false;
		do
		{
			ch = getch();
			switch (ch)
			{
				case _ESC_KEY:
				case 10://enter

				{
					bOK = true;
				}
				break;
				case KEY_UP:
				{
					menu_driver(m_plusmenu, REQ_UP_ITEM);
					wrefresh(m_wins[2]);
					nIndex = item_index(current_item(m_plusmenu));
					PrintPlugin(m_PluginMgr.GetDesc(nIndex));
				}
				break;
				case KEY_DOWN:
				{
					menu_driver(m_plusmenu, REQ_DOWN_ITEM);
					wrefresh(m_wins[2]);
					nIndex = item_index(current_item(m_plusmenu));
					PrintPlugin(m_PluginMgr.GetDesc(nIndex));
				}
				break;
				default:
				break;
			}
		}while (!bOK);
		if (10 == ch)
		{
			nIndex = item_index(current_item(m_plusmenu));
			m_bPainLock = true;
			echo();
			curs_set(2);
			def_prog_mode();
			endwin();
			m_PluginMgr.Run(nIndex);
			reset_prog_mode();
			noecho();
			curs_set(0);
			refresh();
			m_bPainLock = false;
		}
		unpost_menu(m_plusmenu);
		ClearLrc();
		post_menu(m_menu);
		wrefresh(m_wins[2]);
		m_bPluginLock = false;
		RefreshLrc(m_fTPos, true);
	}
	break;
	case eUCMute:
	{
		mvwaddstr(m_wins[1], 5, 1, "[ |< ]  [PAUS]  [STOP]  [ >| ]  [mute]");
		wrefresh(m_wins[1]);
		SendMgrMsg(MSG_MUTE,0,0);
		usleep(200*1000);
		mvwaddstr(m_wins[1], 5, 1, "[ |< ]  [PAUS]  [STOP]  [ >| ]  [MUTE]");
		wrefresh(m_wins[1]);
	}
	break;
	case eUCMode:
	{
		m_bModeLock = true;
		post_menu(m_modemenu);
		wrefresh(m_wins[2]);
		int ch;
		bool bOK = false;
		do
		{
			ch = getch();
			switch (ch)
			{
				case _ESC_KEY:
				case 10://enter

				{
					bOK = true;
				}
				break;
				case KEY_RIGHT:
				{
					menu_driver(m_modemenu, REQ_RIGHT_ITEM);
					wrefresh(m_wins[2]);
				}
				break;
				case KEY_LEFT:
				{
					menu_driver(m_modemenu, REQ_LEFT_ITEM);
					wrefresh(m_wins[2]);
				}
				break;
				default:
				break;
			}
		}while (!bOK);
		if (10==ch)
		{
			m_nCurMode = item_index(current_item(m_modemenu));
			mvwprintw(m_wins[2], m_nLINES-STD_HEG-4, 1, "[%s]",
					m_strMode[m_nCurMode].c_str());
		}
		unpost_menu(m_modemenu);
		wrefresh(m_wins[2]);
		m_bModeLock = false;
	}
	break;
	case eUCPrev:
	{
		int nSize = m_vctMenu.size();
		if (nSize <= 0)
		{
			break;
		}
		mvwaddstr(m_wins[1], 5, 1, "[|<  ]  [PAUS]  [STOP]  [ >| ]  [MUTE]");
		wrefresh(m_wins[1]);
		MenuToSel(m_nCurSel);
		m_nCurSel --;
		if (m_nCurSel < 0)
		{
			m_nCurSel = nSize-1;
			menu_driver(m_menu, REQ_LAST_ITEM);
		}
		else
		{
			menu_driver(m_menu, REQ_UP_ITEM);
		}
		SendMgrMsg(MSG_FILE,0,(LPARAM)m_vctMenu[m_nCurSel].strPath.c_str());
		wrefresh(m_wins[2]);
		usleep(200*1000);
		mvwaddstr(m_wins[1], 5, 1, "[ |< ]  [PAUS]  [STOP]  [ >| ]  [MUTE]");
		wrefresh(m_wins[1]);

	}
	break;
	case eUCNext:
	{
		int nSize = m_vctMenu.size();
		if (nSize <= 0)
		{
			break;
		}
		mvwaddstr(m_wins[1], 5, 1, "[ |< ]  [PAUS]  [STOP]  [  >|]  [MUTE]");
		wrefresh(m_wins[1]);
		MenuToSel(m_nCurSel);
		m_nCurSel ++;
		if (m_nCurSel >= nSize)
		{
			m_nCurSel = 0;
			menu_driver(m_menu, REQ_FIRST_ITEM);
		}
		else
		{
			menu_driver(m_menu, REQ_DOWN_ITEM);
		}
		SendMgrMsg(MSG_FILE,0,(LPARAM)m_vctMenu[m_nCurSel].strPath.c_str());
		wrefresh(m_wins[2]);
		usleep(200*1000);
		mvwaddstr(m_wins[1], 5, 1, "[ |< ]  [PAUS]  [STOP]  [ >| ]  [MUTE]");
		wrefresh(m_wins[1]);
	}
	break;
	case eUCPlay:
	{
		if (m_bPlaying)
		{
			m_bPause = !m_bPause;
			if (m_bPause)
			{
				mvwaddstr(m_wins[1], 5, 1, "[ |< ]  [paus]  [STOP]  [ >| ]  [MUTE]");
				wrefresh(m_wins[1]);
				SendMgrMsg(MSG_PLAY,0,0);
				usleep(200*1000);
				mvwaddstr(m_wins[1], 5, 1, "[ |< ]  [PLAY]  [STOP]  [ >| ]  [MUTE]");
				wrefresh(m_wins[1]);
			}
			else
			{
				mvwaddstr(m_wins[1], 5, 1, "[ |< ]  [play]  [STOP]  [ >| ]  [MUTE]");
				wrefresh(m_wins[1]);
				SendMgrMsg(MSG_PLAY,0,0);
				usleep(200*1000);
				mvwaddstr(m_wins[1], 5, 1, "[ |< ]  [PAUS]  [STOP]  [ >| ]  [MUTE]");
				wrefresh(m_wins[1]);
			}

		}
		else
		{
			mvwaddstr(m_wins[1], 5, 1, "[ |< ]  [paus]  [STOP]  [ >| ]  [MUTE]");
			wrefresh(m_wins[1]);
			SendMgrMsg(MSG_PLAY,0,0);
			usleep(200*1000);
			mvwaddstr(m_wins[1], 5, 1, "[ |< ]  [PAUS]  [STOP]  [ >| ]  [MUTE]");
			wrefresh(m_wins[1]);
		}

	}
	break;
	case eUCStop:
	{
		m_bPause = true;
		mvwaddstr(m_wins[1], 5, 1, "[ |< ]  [PAUS]  [stop]  [ >| ]  [MUTE]");
		wrefresh(m_wins[1]);
		SendMgrMsg(MSG_STOP,0,0);
		usleep(200*1000);
		mvwaddstr(m_wins[1], 5, 1, "[ |< ]  [PLAY]  [STOP]  [ >| ]  [MUTE]");
		wrefresh(m_wins[1]);
	}
	break;
	case eUCList:
	{
		m_bFileLock = true;
		ClearLrc();
		m_nFileType = 1;
		CreateFileMenu(m_wins[3]);
		post_menu(m_filemenu);
		mvwhline(m_wins[3],1, 1, ' ', m_nCOLS-STD_WID-2-2);
		mvwprintw(m_wins[3], 1, 1, "dir:%s", m_FileMgr.GetPath(-1).c_str());
		wrefresh(m_wins[3]);
		int nIndex = 0;
		int ch, chTmp;
		bool bOK = false;
		bool bNext = true;
		do
		{
			chTmp = 0;
			bOK = false;
			bNext = true;
			do
			{

				ch = getch();
				switch (ch)
				{
					case _ESC_KEY:
					{
						bNext = false;
					}
					case ' ':
					case KEY_BACKSPACE:
					case 10://enter

					{
						bOK = true;
					}
					break;
					case KEY_UP:
					{
						menu_driver(m_filemenu, REQ_UP_ITEM);
						wrefresh(m_wins[3]);
					}
					break;
					case KEY_DOWN:
					{
						menu_driver(m_filemenu, REQ_DOWN_ITEM);
						wrefresh(m_wins[3]);
					}
					break;
					default:
					{
						if (ch >= 'a' && ch <= 'z')
						{
							if (chTmp == ch)
							{
								menu_driver(m_filemenu, REQ_NEXT_MATCH);
							}
							else
							{
								menu_driver(m_filemenu, REQ_CLEAR_PATTERN);
								menu_driver(m_filemenu, ch);
								chTmp = ch;
							}
							wrefresh(m_wins[3]);
						}
					}
					break;
				}
			}while (!bOK);
			if (' ' == ch)
			{
				m_bPainLock = true;

				mvwaddstr(m_wins[2], m_nLINES-STD_HEG-4-1, 1, "save as:");
				wrefresh(m_wins[2]);

				echo();
				curs_set(2);
				char str[64];
				mvwgetstr(m_wins[2],m_nLINES-STD_HEG-4-1, 9,str);
				curs_set(0);
				noecho();

				if (strlen(str) > 0)
				{
					char* pType = strstr(str,".lst");
					//					log ("len:0x%x,%d,%d\n",pType,(pType - str),strlen(str));
					if ((pType && (size_t)(pType - str) == (strlen(str) - 4))
							|| 0 == MsgBox("Confirm","Save as unkown type?(*.lst)")
					)
					{
						std::string strPath = m_FileMgr.GetPath(-1);
						strPath += str;
						SaveList(strPath.c_str());
					}
					bNext = false;
				}

				mvwhline(m_wins[2],m_nLINES-STD_HEG-4-1, 1, ' ', STD_WID-2);
				wrefresh(m_wins[2]);

				if (strlen(str) > 0)
				{
					unpost_menu(m_filemenu);
					CreateFileMenu(m_wins[3]);
					post_menu(m_filemenu);
				}

				m_bPainLock = false;
			}
			else if (10 == ch || KEY_BACKSPACE == ch)
			{
				nIndex = KEY_BACKSPACE == ch?
				m_FileMgr.FindIndex("../"):item_index(current_item(m_filemenu));
				if (m_FileMgr.SetSel(nIndex))
				{
					menu_driver(m_filemenu, REQ_CLEAR_PATTERN);
					unpost_menu(m_filemenu);
					CreateFileMenu(m_wins[3]);
					post_menu(m_filemenu);
					mvwhline(m_wins[3],1, 1, ' ', m_nCOLS-STD_WID-2-2);
					mvwprintw(m_wins[3], 1, 1, "dir:%s", m_FileMgr.GetPath(-1).c_str());
					wrefresh(m_wins[3]);
				}
				else if (10 == ch)
				{
					CreateListMenu(m_wins[2], m_FileMgr.GetPath(nIndex).c_str());
					bNext = false;
				}
			}
		}while (bNext);
		menu_driver(m_filemenu, REQ_CLEAR_PATTERN);
		unpost_menu(m_filemenu);
		mvwhline(m_wins[3],1, 1, ' ', m_nCOLS-STD_WID-2-2);
		wrefresh(m_wins[3]);
		m_bFileLock = false;
		RefreshLrc(m_fTPos, true);
	}
	break;
	case eUCLrcd:
	{
		m_bFileLock = true;
		ClearLrc();
		m_nFileType = 2;
		CreateFileMenu(m_wins[3]);
		post_menu(m_filemenu);
		mvwhline(m_wins[3],1, 1, ' ', m_nCOLS-STD_WID-2-2);
		mvwprintw(m_wins[3], 1, 1, "dir:%s", m_FileMgr.GetPath(-1).c_str());
		wrefresh(m_wins[3]);
		int nIndex = 0;
		int ch, chTmp;
		bool bOK = false;
		bool bNext = true;
		do
		{
			chTmp = 0;
			bOK = false;
			bNext = true;
			do
			{
				ch = getch();
				switch (ch)
				{
					case _ESC_KEY:
					{
						bNext = false;
					}
					case ' ':
					case KEY_BACKSPACE:
					case 10://enter

					{
						bOK = true;
					}
					break;
					case KEY_UP:
					{
						menu_driver(m_filemenu, REQ_UP_ITEM);
						wrefresh(m_wins[3]);
					}
					break;
					case KEY_DOWN:
					{
						menu_driver(m_filemenu, REQ_DOWN_ITEM);
						wrefresh(m_wins[3]);
					}
					break;
					default:
					{
						if (ch >= 'a' && ch <= 'z')
						{
							if (chTmp == ch)
							{
								menu_driver(m_filemenu, REQ_NEXT_MATCH);
							}
							else
							{
								menu_driver(m_filemenu, REQ_CLEAR_PATTERN);
								menu_driver(m_filemenu, ch);
								chTmp = ch;
							}
							wrefresh(m_wins[3]);
						}
					}
					break;
				}
			}while (!bOK);
			if ( ' ' == ch)
			{
				nIndex = item_index(current_item(m_filemenu));

				m_bPainLock = true;
				std::string strDir;
				if (m_FileMgr.GetType(nIndex) & DT_DIR)
				{
					strDir = m_FileMgr.GetPath(nIndex);
				}
				else
				{
					strDir = m_FileMgr.GetPath(-1);
				}
				if (strDir.rfind('/')!=strDir.length()-1)
				{
					strDir += "/";
				}
				if (0 == MsgBox("[Save Lrc Dir as :]",strDir.c_str()))
				{
					m_strLrcDir = strDir;
				}
				bNext = false;
				m_bPainLock = false;

			}
			else if (10 == ch || KEY_BACKSPACE == ch)
			{
				nIndex = KEY_BACKSPACE == ch?
				m_FileMgr.FindIndex("../"):item_index(current_item(m_filemenu));
				if (m_FileMgr.SetSel(nIndex))
				{
					menu_driver(m_filemenu, REQ_CLEAR_PATTERN);
					unpost_menu(m_filemenu);
					CreateFileMenu(m_wins[3]);
					post_menu(m_filemenu);
					mvwhline(m_wins[3],1, 1, ' ', m_nCOLS-STD_WID-2-2);
					mvwprintw(m_wins[3], 1, 1, "dir:%s", m_FileMgr.GetPath(-1).c_str());
					wrefresh(m_wins[3]);
				}
			}
		}while (bNext);
		menu_driver(m_filemenu, REQ_CLEAR_PATTERN);
		unpost_menu(m_filemenu);
		mvwhline(m_wins[3],1, 1, ' ', m_nCOLS-STD_WID-2-2);
		wrefresh(m_wins[3]);
		m_bFileLock = false;
		RefreshLrc(m_fTPos, true);
	}
	break;
	default:
	break;
}
}

void CUIMgr::CreateFileMenu(WINDOW* win)
{
	DestroyFileMenu();
	wrefresh(win);
	m_FileMgr.Load(m_nFileType);
	int nSize = m_FileMgr.GetSize();
	m_fileitems = (ITEM **)calloc(nSize+1, sizeof(ITEM *));
	for (int i = 0; i < nSize; ++i)
	{
		m_fileitems[i] = new_item(m_FileMgr.GetName(i), m_FileMgr.GetDesc(i));
	}
	m_fileitems[nSize] = new_item(NULL, NULL);

	m_filemenu = new_menu((ITEM **)m_fileitems);

	menu_opts_off(m_filemenu, O_ONEVALUE);
	menu_opts_off(m_filemenu, O_SHOWDESC);

	set_menu_win(m_filemenu, win);
	set_menu_sub(m_filemenu, derwin(win, m_nLINES-6, m_nCOLS-STD_WID-2-2, 2, 1));
	set_menu_format(m_filemenu, m_nLINES-6, 1);

	//	set_menu_mark(m_filemenu, " > ");
}

void CUIMgr::CreatePluginMenu(WINDOW* win)
{
	m_PluginMgr.Load();
	int nSize = m_PluginMgr.GetSize();
	m_plusitems = (ITEM **)calloc(nSize+1, sizeof(ITEM *));
	for (int i = 0; i < nSize; ++i)
	{
		m_plusitems[i] = new_item(m_PluginMgr.GetName(i), NULL);
	}
	m_plusitems[nSize] = new_item(NULL, NULL);

	m_plusmenu = new_menu((ITEM **)m_plusitems);

	set_menu_win(m_plusmenu, win);
	set_menu_sub(m_plusmenu, derwin(win, m_nLINES-STD_HEG-7, STD_WID-2, 2, 1));
	set_menu_format(m_plusmenu, m_nLINES-STD_HEG-7, 1);

	set_menu_mark(m_plusmenu, " > ");
}

void CUIMgr::CreateMsgMenu(WINDOW* win)
{
	m_strMsgItem[0] = "[Yes]";
	m_strMsgItem[1] = "[No ]";
	m_msgitems = (ITEM **)calloc(3, sizeof(ITEM *));
	for (int i = 0; i < 2; ++i)
	{
		m_msgitems[i] = new_item(m_strMsgItem[i].c_str(), NULL);
	}
	m_msgitems[2] = new_item(NULL, NULL);

	m_msgmenu = new_menu((ITEM **)m_msgitems);

	set_menu_win(m_msgmenu, win);
	set_menu_sub(m_msgmenu, derwin(win, 1, 16, 5, (m_nCOLS-4 - 16)/2));
	set_menu_format(m_msgmenu, 1, 2);

	set_menu_mark(m_msgmenu, "*");

	post_menu(m_msgmenu);
	wrefresh(win);
}

void CUIMgr::CreateModeMenu(WINDOW* win)
{
	m_strMode[0] = "SING";
	m_strMode[1] = "SILP";
	m_strMode[2] = "MULT";
	m_strMode[3] = "MULP";
	m_strMode[4] = "RAND";
	m_modeitems = (ITEM **)calloc(6, sizeof(ITEM *));
	for (int i = 0; i < 5; ++i)
	{
		m_modeitems[i] = new_item(m_strMode[i].c_str(), NULL);
	}
	m_modeitems[5] = new_item(NULL, NULL);

	m_modemenu = new_menu((ITEM **)m_modeitems);

	set_menu_win(m_modemenu, win);
	set_menu_sub(m_modemenu,
			derwin(win, 1, STD_WID-2, m_nLINES-STD_HEG-2 -3, 1));
	set_menu_format(m_modemenu, 1, 5);

	set_menu_mark(m_modemenu, ">");
}

void CUIMgr::CreateListMenu(WINDOW* win, const char* lpszFile)
{
	DestroyListMenu(0==lpszFile ? false : true);
	wrefresh(win);
	m_nCurSel = 0;
	if (lpszFile)
	{
		std::ifstream infile(lpszFile);
		std::string strTmp, strMenu;
		size_t nRPos = 0;
		while (getline(infile, strTmp))
		{
			if ((nRPos =strTmp.rfind('/')) != std::string::npos)
			{
				strMenu = strTmp.substr(nRPos+1, strTmp.length()-nRPos-1);
			}
			else
			{
				strMenu = strTmp;
			}
			MENUITEM item;
			item.strName = strMenu;
			item.strPath = strTmp;
			m_vctMenu.push_back(item);
		}
	}

	int nSize = m_vctMenu.size();
	if (nSize <= 0)
	{
		return;
	}
	m_items = (ITEM **)calloc(nSize+1, sizeof(ITEM *));

	for (int i = 0; i < nSize; ++i)
	{
		m_items[i] = new_item(m_vctMenu[i].strName.c_str(), NULL);
	}
	m_items[nSize] = new_item(NULL, NULL);

	m_menu = new_menu((ITEM **)m_items);

	set_menu_win(m_menu, win);
	set_menu_sub(m_menu, derwin(win, m_nLINES-STD_HEG-7, STD_WID-2, 2, 1));
	set_menu_format(m_menu, m_nLINES-STD_HEG-7, 1);

	set_menu_mark(m_menu, " > ");

	post_menu(m_menu);
	wrefresh(win);
}

void CUIMgr::DestroyListMenu(bool bClear)
{
	if (m_menu)
	{
		int nSize = item_count(m_menu);
		unpost_menu(m_menu);
		free_menu(m_menu);
		for (int i = 0; i < nSize; ++i)
		{
			free_item(m_items[i]);
		}
		free(m_items);
		if (bClear)
		{
			m_vctMenu.clear();
		}
		m_menu = NULL;
	}
}

void CUIMgr::DestroyFileMenu()
{
	if (m_filemenu)
	{
		int nSize = item_count(m_filemenu);
		unpost_menu(m_filemenu);
		free_menu(m_filemenu);
		for (int i = 0; i < nSize; ++i)
		{
			free_item(m_fileitems[i]);
		}
		free(m_fileitems);
		m_FileMgr.Clear();
		m_filemenu = NULL;
	}
}

void CUIMgr::MenuToSel(int nSel)
{
	if (!m_menu)
	{
		return;
	}
	int nIndex = item_index(current_item(m_menu));
	if (nIndex > nSel)
	{
		while (nIndex != nSel)
		{
			menu_driver(m_menu, REQ_UP_ITEM);
			nIndex --;
		}
	}
	else if (nIndex < nSel)
	{
		while (nIndex != nSel)
		{
			menu_driver(m_menu, REQ_DOWN_ITEM);
			nIndex ++;
		}
	}
}

std::string CUIMgr::Float2Time(float fTotalSec)
{
	int nTotalSec = (int)fTotalSec;
	int nSec = nTotalSec % 60;
	int nMin = (nTotalSec - nSec)/60;
	int nTmp;
	int nHour = 0;
	if (nMin >= 60)
	{
		nTmp = nMin % 60;
		nHour = (nMin - nTmp) / 60;
		nMin = nTmp;
	}
	char szTmp[32];
	sprintf(szTmp, "%.1d:%.2d:%.2d", nHour, nMin, nSec);
	std::string s(szTmp);
	return s;
}

bool CUIMgr::CheckAutoNext()
{
	bool bRet = false;
	double fSec = difftime(m_tRTime, m_tSTime);
	int nMode = m_nCurMode;
	if (m_bMsgBoxLock) //avoid flower screen.
	{
		nMode = 0;
	}
	if (fSec > 2.0f || fSec < -2.0f)
	{
		bRet = true;
		switch (nMode)
		{
		case 0:
		{
			m_bPlaying = false;
		}
			break;
		case 1:
		{
			SendMgrMsg(MSG_FILE,0,(LPARAM)m_vctMenu[m_nCurSel].strPath.c_str());
		}
			break;
		case 2:
		{
			int nSize = m_vctMenu.size();
			if (m_nCurSel == nSize - 1)
			{
				m_bPlaying = false;
			}
			else
			{
				Command(eUCNext);
			}
		}
			break;
		case 3:
		{
			Command(eUCNext);
		}
			break;
		case 4:
		{
			int nSize = m_vctMenu.size();
			srand(time(NULL));
			int nTmp;
			if (nSize>1)
			{
				do
				{
					nTmp = rand() % m_vctMenu.size();
				} while (nTmp == m_nCurSel);
			}
			else
			{
				nTmp = 0;
			}
			m_nCurSel = nTmp;
			MenuToSel(m_nCurSel);
			wrefresh(m_wins[2]);
			SendMgrMsg(MSG_FILE,0,(LPARAM)m_vctMenu[m_nCurSel].strPath.c_str());
		}
			break;
		default:
			break;
		}
		m_tRTime = m_tSTime;
	}
	return bRet;
}

void CUIMgr::PrintLrc(const char* lpszStr)
{
	std::string str = lpszStr;
	size_t nPos = 0;
	size_t nStart = 0;
	size_t nLen = 0;
	int nLine = 3;
	size_t nPrintWid = m_nCOLS-STD_WID-4;
	int nOffset = 0;
	int nMid = (m_nLINES-8)/2; // in fact the index. base 0.
	int nCnt1 = 0;
	int nCnt2 = 0;

	int nSize = strlen(lpszStr);
	wchar_t* wcs = new wchar_t[nSize];
	memset(wcs, 0, nSize*sizeof(wchar_t));
	mbstowcs(wcs, lpszStr, nSize-1);
	std::wstring wstr(wcs);
	delete[] wcs;

#ifndef HAVE_MVWADDWSTR
	char* cs = new char[nSize*2];
#endif

	double fPatch = 1.0f;
	size_t nPatchLen = 0;
	while ((nPos=wstr.find(L'\n', nStart))!=std::wstring::npos && nLine <= m_nLINES-6)
	{
		nCnt1++;
		nLen = nPos - nStart;
		// patch begin
		fPatch = 1;
		if (nLen > 0)
		{
			char* cs2 = new char[nLen*2];
			std::wstring wstr2 = wstr.substr(nStart,nLen);
			wcstombs(cs2,wstr2.c_str(),nLen*2);
			if (strlen(cs2) != wstr2.length())
			{
				fPatch = ((double)wstr2.length())/((double)strlen(cs2));
			}
			delete []cs2;
		}
		nPatchLen = (size_t)(((double)nPrintWid)*fPatch);
		// patch end
		if (nLen > nPatchLen)
		{
			nLen = nPatchLen;
			wstr.insert(nStart + nLen, L"\n");
			nPos = nStart + nLen;
			nCnt2++;
			if (nCnt1-nCnt2<nMid)
			{
				nOffset ++;
			}
		}
		mvwhline(m_wins[3],nLine, 1, ' ', m_nCOLS-STD_WID-4);
		if (nLen > 0)
		{
#ifndef HAVE_MVWADDWSTR
			memset(cs,0,nSize*2);
			wcstombs(cs,wstr.substr(nStart,nLen).c_str(),nSize*2);
			mvwaddstr(m_wins[3], nLine, 1, cs);
#else
			mvwaddwstr(m_wins[3], nLine, 1, wstr.substr(nStart,nLen).c_str());
#endif
		}
		nLine ++;
		nStart = nPos+1;
	}
#ifndef HAVE_MVWADDWSTR
	delete []cs;
#endif

	if (0 < m_nLrcLight)
	{
		mvwchgat(m_wins[3],nMid+3+nOffset,1,m_nCOLS-STD_WID-4, GetLrcLight(), 0, NULL);
	}
	wrefresh(m_wins[3]);
}

attr_t CUIMgr::GetLrcLight()
{
	if (1 == m_nLrcLight)
	{
		return A_STANDOUT;
	}
	else if (2 == m_nLrcLight)
	{
		return A_UNDERLINE;
	}
	else if (3 == m_nLrcLight)
	{
		return A_REVERSE;
	}
	else if (4 == m_nLrcLight)
	{
		return A_BLINK;
	}
	else if (5 == m_nLrcLight)
	{
		return A_DIM;
	}
	else if (6 == m_nLrcLight)
	{
		return A_BOLD;
	}
	else if (7 == m_nLrcLight)
	{
		return A_PROTECT;
	}
	else if (8 == m_nLrcLight)
	{
		return A_INVIS;
	}
	else if (9 == m_nLrcLight)
	{
		return A_ALTCHARSET;
	}
	else if (10 == m_nLrcLight)
	{
		return A_CHARTEXT;
	}
	return A_NORMAL;
}

void CUIMgr::PrintPlugin(const char* lpszStr)
{
	ClearLrc();
	if (!lpszStr)
	{
		return;
	}
	std::string str = lpszStr;
	size_t nPos = 0;
	size_t nStart = 0;
	size_t nLen = 0;
	int nLine = 3;
	size_t nPrintWid = m_nCOLS-STD_WID-4;

	int nSize = strlen(lpszStr);
	wchar_t* wcs = new wchar_t[nSize];
	memset(wcs, 0, nSize*sizeof(wchar_t));
	mbstowcs(wcs, lpszStr, nSize-1);
	std::wstring wstr(wcs);
	delete[] wcs;
#ifndef HAVE_MVWADDWSTR
	char* cs = new char[nSize*2];
#endif
	double fPatch = 1.0f;
	size_t nPatchLen = 0;
	while ((nPos=wstr.find(L'\n', nStart))!=std::wstring::npos && nLine <= m_nLINES-6)
	{
		nLen = nPos - nStart;
		// patch begin
		fPatch = 1;
		if (nLen > 0)
		{
			char* cs2 = new char[nLen*2];
			std::wstring wstr2 = wstr.substr(nStart,nLen);
			wcstombs(cs2,wstr2.c_str(),nLen*2);
			if (strlen(cs2) != wstr2.length())
			{
				fPatch = ((double)wstr2.length())/((double)strlen(cs2));
			}
			delete []cs2;
		}
		nPatchLen = (size_t)(((double)nPrintWid)*fPatch);
		// patch end		
		if (nLen > nPatchLen)
		{
			nLen = nPatchLen;
			wstr.insert(nStart + nLen, L"\n");
			nPos = nStart + nLen;
		}
		mvwhline(m_wins[3],nLine, 1, ' ', m_nCOLS-STD_WID-4);
		if (nLen > 0)
		{
#ifndef HAVE_MVWADDWSTR
			memset(cs,0,nSize*2);
			wcstombs(cs,wstr.substr(nStart,nLen).c_str(),nSize*2);
			mvwaddstr(m_wins[3], nLine, 1, cs);
#else
			mvwaddwstr(m_wins[3], nLine, 1, wstr.substr(nStart,nLen).c_str());
#endif
		}
		nLine ++;
		nStart = nPos+1;
	}
#ifndef HAVE_MVWADDWSTR
	delete []cs;
#endif
	wrefresh(m_wins[3]);
}

int CUIMgr::SendMgrMsg(const char* lpszMsg, WPARAM wParam, LPARAM lParam)
{
	if (0 == strcmp(lpszMsg, MSG_FILE))
	{
		if (lParam)
		{
			wParam = (WPARAM)m_nVolum;
			ClearLrc();
			bool bRet = false;
			std::string str = GetLrcPath((const char*)lParam, 0);
			if (!str.empty())
			{
				m_nLastLrcIndex = -1;
				bRet = m_LrcMgr.LoadFile(str.c_str());
			}
			if (!bRet)
			{
				str = GetLrcPath((const char*)lParam, 1);
				if (!str.empty())
				{
					m_nLastLrcIndex = -1;
					bRet = m_LrcMgr.LoadFile(str.c_str());
				}
			}
			if (!bRet)
			{
				str = GetLrcPath((const char*)lParam, 2);
				if (!str.empty())
				{
					m_nLastLrcIndex = -1;
					bRet = m_LrcMgr.LoadFile(str.c_str());
				}
			}
			if (!bRet)
			{
				str = (const char*)lParam;
				std::string strFileName;
				size_t nRPos;
				if ((nRPos =str.rfind('/')) != std::string::npos)
				{
					strFileName = str.substr(nRPos+1, str.length()-nRPos-1); //file name
				}
				OnMsg(MSG_EVENT,(WPARAM)ONLRCNONE,(LPARAM)strFileName.c_str());
			}
		}
	}
	else if (0 == strcmp(lpszMsg, MSG_URL))
	{
		wParam = (WPARAM)m_nVolum;
	}
	int nRet = CManager::SendMgrMsg(lpszMsg, wParam, lParam);
	return nRet;
}

std::string CUIMgr::GetLrcPath(const char* lpszFile, int nType)
{
	//0:LrcDir 1:music path 2:Lrc dir
	std::string str;
	std::string strFileName;
	if (nType >= 0 || nType <= 2)
	{
		str = lpszFile;
		size_t nRPos;
		if ((nRPos =str.rfind('/')) != std::string::npos)
		{
			strFileName = str.substr(nRPos+1, str.length()-nRPos-1); //file name
		}
		str = "";
		if (0 == nType)
		{
			if (!m_strLrcDir.empty())
			{
				str = m_strLrcDir + strFileName; //path but no lrc.
			}
		}
		else if (1 == nType)
		{
			str = lpszFile;
		}
		else if (2 == nType)
		{
			str = getpath("Lrc/");
			str += strFileName;
		}
		//chg to .lrc file.
		nRPos = str.rfind('.');
		if (nRPos != std::string::npos)
		{
			str = str.substr(0, nRPos);
			str += ".lrc";
		}
	}
	return str;
}

void CUIMgr::RefreshLrc(float fPos, bool bForce)
{
	if (m_LrcMgr.IsEmpty() || m_bPluginLock || m_bFileLock || m_bAboutLock
			|| m_bMsgBoxLock)
	{
		return;
	}
	int nIndex = m_LrcMgr.GetIndex(fPos);
	if (nIndex != m_nLastLrcIndex || bForce)
	{
		std::string str = m_LrcMgr.GetStr(nIndex);
		PrintLrc(str.c_str());
		m_nLastLrcIndex = nIndex;
	}
}

void CUIMgr::ClearLrc()
{
	int nLine = 3;
	while (nLine <= m_nLINES-6)
	{
		mvwhline(m_wins[3],nLine, 1, ' ', m_nCOLS-STD_WID-4);
		nLine ++;
	}
	wrefresh(m_wins[3]);
}

void CUIMgr::PrintAbout()
{
	m_bAboutLock = true;
	ClearLrc();
	char szAbout[1024];
	sprintf(szAbout,_VER_DESC,_VERSION,"press any key to continue ...\n...");
	PrintPlugin(szAbout);
	getch();
	ClearLrc();
	m_bAboutLock = false;
	RefreshLrc(m_fTPos, true);
}

